import java.io.*;
import java.util.*;

//A bipartite graph.  may create extra bypass nodes if necessary.  keeps
//track of all bypass nodes used.

public class BGraph{

    public BGraph(RGraph rg){
	this.maxDegree = rg.maxDegree();
	int numRGNodes = rg.numNodes();
	int numNodes = 0;

	Vector vLeftNodes = new Vector();
	Vector vRightNodes = new Vector();

	RGNode arrRGNodes[] = rg.getNodes();

	rgnToBN = new RGNtoBNMapper(numRGNodes);

	//place all the nodes in the graph
	for(int i=0;i<numRGNodes;i++){
	    RGNode rgn = arrRGNodes[i];

	    insertRight(rgn,numNodes,vRightNodes);
	    numNodes++;
	    if(rgn.degree()==maxDegree){
		insertLeft(rgn,numNodes,vLeftNodes);
		numNodes++;
	    }
	}

	//put all the edges in the graph
	RGEdge arrRGEdges[] = rg.getEdges();
	for(int i=0;i<arrRGEdges.length;i++){
	    insertEdge(arrRGEdges[i]);
	}
	arrLeftNodes = Node.vectorToArrNodes(vLeftNodes);
	arrRightNodes = Node.vectorToArrNodes(vRightNodes);
    }


    public String toString(){
	String s = "BGraph\n";
	s = s + "Left Side\n" + MyIOUtils.arrToString(arrLeftNodes,"\n");
	s = s + "\nRight Side\n" + MyIOUtils.arrToString(arrRightNodes,"\n");
	return s;
    }

//     //PRE: leftP is true iff node is to be inserted on left side.
//     private void insertNode(RGNode rgn, boolean leftP, int nodeID, Vector vNodes){
// 	Node n = new Node(rgn,nodeID);
// 	int rgnNum = rgn.getUniqueNumber();
// 	if(leftP){
// 	    rgnToBN.insertLeft(rgnNum,n);
// 	    vNodes.addElement(n);
// 	} else {
// 	    rgnToBN.insertRight(rgnNum,n);
// 	    vNodes.addElement(n);
// 	}
//     }

    //PRE: rgn has maximum degree
    //POST: inserts rgn onto the left side of the bipartite graph
    private void insertLeft(RGNode rgn, int numNodes, Vector vLeftNodes){
	Node n = new Node(rgn,numNodes);
	int rgnNum = rgn.getUniqueNumber();
	rgnToBN.insertLeft(rgnNum,n);
	vLeftNodes.addElement(n);
    }

    private void insertRight(RGNode rgn, int numNodes, Vector vRightNodes){
	Node n = new Node(rgn,numNodes);
	int rgnNum = rgn.getUniqueNumber();
	rgnToBN.insertRight(rgnNum,n);
	vRightNodes.addElement(n);
    }


//     private Node getLeftNodeFromRGN(RGNode rgn){
// 	int rgnNum = rgn.getUniqueNumber();
// 	Node n = arrRGNtoLeftNodes[rgnNum];
// 	return n;
//     }
    
//     //PRE: rgn is a RGNode which is contained on the right side of the graph
//     //POST: RES equals the node that contains rgn on the right side
//     private Node getRightNodeFromRGN(RGNode rgn){
// 	int rgnNum = rgn.getUniqueNumber();
// 	Node n = arrRGNtoRightNodes[rgnNum];
// 	Assert.POST(n!=null);
// 	return n;
//     }

    //PRE: source and sink are nonnull
    //POST: adds the edge to the nodes source and sink
    private void addEdge(Node source, Node sink){
	Assert.PRE(source!=null&sink!=null);
	Edge e = new Edge(source, sink);
	source.addEdge(e);
	sink.addEdge(e);
    }

    private void insertEdge(RGEdge rge){
	RGNode source = rge.source();
	RGNode sink = rge.sink();
	Node nLeft = rgnToBN.getLeftNodeFromRGN(source);
	Node nRight = null;

	//Debug.trace2(source,sink);
	if(nLeft!=null){
	    nRight = rgnToBN.getRightNodeFromRGN(sink);
	    Assert.PRE(nRight!=null);
	    addEdge(nLeft,nRight);
	}
	nLeft = rgnToBN.getLeftNodeFromRGN(sink);
	if(nLeft!=null){
	    nRight = rgnToBN.getRightNodeFromRGN(source);
	    Assert.PRE(nRight!=null);
	    addEdge(nRight,nLeft);
	}
    }


    //POST: returns the ith node on the left side of the graph
    public Node getLeftNode(int i){
	return arrLeftNodes[i];
    }

    public Node getRightNode(int i){
	return arrRightNodes[i];
    }


    //returns the degree of all the vertices on the lhs of the graph.
    public int maxDegree(){
	return maxDegree;
    }
    
    public int numNodes(){
	return arrLeftNodes.length+arrRightNodes.length;
    }

    //POST: gives number of nodes on left hand side
    public int sizeLeft(){
	return arrLeftNodes.length;
    }

    public int sizeRight(){
	return arrRightNodes.length;
    }


    public RGNtoBNMapper getRGNtoBNMapper(){
	return rgnToBN;
    }

    public Node[] getLeftNodes(){
	return arrLeftNodes;
    }

    RGNtoBNMapper rgnToBN;
    Node arrLeftNodes[];
    Node arrRightNodes[];    
    int maxDegree;
};
