import java.io.*;
import java.util.*;

public class Node{


    public Node(RGNode rgn, int id){
	this.uniqueID = id;
	this.rgn = rgn;
	this.vEdges = new Vector();
    }


    public boolean isBypass(){
	return rgn.isBypass();
    }

    public int degree(){
	return vEdges.size();
    }

    //POST: returns an array of all the neighboring nodes.  May contain duplicates
    public Node[] getNeighbors(){
	Node arrN[] = new Node[vEdges.size()];
	for(int i=0;i<vEdges.size();i++){
	    Edge currE = (Edge)vEdges.elementAt(i);
	    Node nNeighbor = currE.getNeighbor(this);
	    Assert.PRE(nNeighbor!=null);
	    arrN[i] = nNeighbor;
	}
	return arrN;
    }

    public Edge[] getEdges(){
	Edge arrE[] = new Edge[vEdges.size()];
	for(int i=0;i<vEdges.size();i++){
	    arrE[i] = (Edge)vEdges.elementAt(i);
	}
	return arrE;
    }

    public int getUniqueNumber(){
	return uniqueID;
    }

    public void addEdge(Edge e){
	vEdges.addElement(e);
    }

    //PRE: there is an edge connecting this to n
    //POST: RES equals that edge
    public Edge getConnectingEdge(Node n){
	for(int i=0;i<vEdges.size();i++){
	    Edge currE = (Edge) vEdges.elementAt(i);
	    Node currNhbr = currE.getNeighbor(this);
	    if(currNhbr==n){
		return currE;
	    }
	}
	Assert.POST(false);
	return null;
    }

    public RGNode rgnode(){
	return rgn;
    }

    public String toString(){
	String sRes = "Node: " + Integer.toString(uniqueID) + "\n";
	int rgnID = rgn.getUniqueNumber();
	sRes = sRes + "Contains: " + Integer.toString(rgnID) + "\n";
	//now include info about the edges
	for(int i=0;i<vEdges.size();i++){
	    Edge edge = (Edge)vEdges.elementAt(i);
	    Node nNeighbor = edge.getNeighbor(this);
	    int nNum = nNeighbor.getUniqueNumber();
	    sRes = sRes + Integer.toString(nNum) + " ";
	}
	return sRes;
    }

    public static Node[] vectorToArrNodes(Vector v){
	Node arrNodes[] = new Node[v.size()];
	for(int i=0;i<v.size();i++){
	    arrNodes[i] = (Node)v.elementAt(i);
	}
	return arrNodes;
    }


    int uniqueID;
    RGNode rgn;
    Vector vEdges;
};
