//this class given a matching, a mapping between nodes in the matching
//and nodes in the request graph and a stack of free bypass nodes,
//computes the edges that should be sent for the given phase and uses
//any necessary bypass nodes.  It is used to update the request graph.

import java.io.*;
import java.util.*;
public class PhaseSchedule{

    //PRE: rgCyclesAndPaths contains a graph giving the cycles and
    //paths connecting the nodes with highest degree; the maximum
    //degree in rgCyclesAndPaths is <= 2.  stRGNFreeBypass contains
    //any free bypass nodes.
    public PhaseSchedule(int numNodesInGraph, RGNode arrMaxNodes[],Stack stRGNFreeBypass){
	Assert.PRE(stRGNFreeBypass!=null);

	CyclesAndPaths cyclesAndPaths = getPathsAndCycles(numNodesInGraph, arrMaxNodes);
	Cycle arrCycles[] = cyclesAndPaths.arrCycles;
	Path arrPaths[] = cyclesAndPaths.arrPaths;
	//Debug.trace(arrPaths);
	//Debug.trace(arrCycles);
	scheduleEdges(arrPaths, arrCycles,stRGNFreeBypass);
	
	Assert.POST(isValid(numNodesInGraph));
    }

    private void addEvenEdgesToVector(Vector v, RGEdge arrRGE[]){
	for(int i=0;i<arrRGE.length;i+=2){
	    RGEdge rge = arrRGE[i];
	    v.addElement(rge);
	}
    }

    private void addEdgesToVector(Vector v, RGEdge arrRGE[]){
	for(int i=0;i<arrRGE.length;i++){
	    RGEdge rge = arrRGE[i];
	    v.addElement(rge);
	}
    }

    public static RGNode []getLeftRGNodes(Matching match){
	Node arrNode[] = match.getLeftNodes();
	RGNode arrRGNode[] = new RGNode[arrNode.length];
	for(int i=0;i<arrNode.length;i++){
	    arrRGNode[i] = arrNode[i].rgnode();
	}
	return arrRGNode;
    }

    public class PathsAndArrVisited{
	public PathsAndArrVisited(Vector vPaths, boolean arrIsVisited[]){
	    this.vPaths = vPaths;
	    this.arrIsVisited = arrIsVisited;
	}

	public Vector vPaths;
	public boolean arrIsVisited[];
    }

    private PathsAndArrVisited getPaths(boolean arrIsVisited[], RGNode arrMaxNodes[]){
	int numNodesInGraph = arrIsVisited.length;
	int numMaxNodes = arrMaxNodes.length;
	Vector vPaths = new Vector();

	for(int i=0;i<numMaxNodes;i++){
	    RGNode firstNode = arrMaxNodes[i];
	    int firstID = firstNode.getUniqueNumber();
	    if(!arrIsVisited[firstID]&&(firstNode.degree()==1)){
		//create a path
		RGEdge arrRGE[] = getCCEdges(numNodesInGraph,firstNode);
		arrIsVisited = markVisited(arrRGE,arrIsVisited,firstNode);
		int numEdges = arrRGE.length;
		Assert.PRE(arrRGE!=null);
		Assert.PRE(numEdges>=1);
		Path path = new Path(firstNode,arrRGE);
		vPaths.addElement(path);
	    }
	}
	PathsAndArrVisited pAV = new PathsAndArrVisited(vPaths,arrIsVisited);
	return pAV;
    }


    //POST:  RES contains arrPaths and arrCycles which are the paths and cycles in the matching
    private CyclesAndPaths getPathsAndCycles(int numNodesInGraph, RGNode arrMaxNodes[]){
	Vector vPaths = new Vector();
	Vector vCycles = new Vector();
	int numMaxNodes = arrMaxNodes.length;
	boolean arrIsVisited[] = MyArrayUtils.getAllFalseArr(numNodesInGraph);

	PathsAndArrVisited pAndAV = getPaths(arrIsVisited,arrMaxNodes);
	vPaths = pAndAV.vPaths;
	arrIsVisited = pAndAV.arrIsVisited;

	for(int i=0;i<numMaxNodes;i++){
	    RGNode firstNode = arrMaxNodes[i];
	    int firstID = firstNode.getUniqueNumber();
	    if(!arrIsVisited[firstID]){
		//create a cycle or a path
		RGEdge arrRGE[] = getCCEdges(numNodesInGraph,firstNode);
		arrIsVisited = markVisited(arrRGE,arrIsVisited,firstNode);
		int numEdges = arrRGE.length;
		Assert.PRE(arrRGE!=null);
		Assert.PRE(numEdges>=1);
		RGEdge rgeLast = arrRGE[numEdges-1];
		if(numEdges>=2&&rgeLast.contains(firstNode)){
		    Cycle cyc = new Cycle(firstNode,arrRGE);
		    vCycles.addElement(cyc);
		}else{
		    //Path path = new Path(firstNode,arrRGE);
		    //vPaths.addElement(path);
		    Assert.POST(false);
		}
	    }
	}
	Path arrPaths[] = vPathsToArrPaths(vPaths);
	Cycle arrCycles[] = vCyclesToArrCycles(vCycles);
	CyclesAndPaths cAndP = new CyclesAndPaths(arrPaths,arrCycles);
	return cAndP;
    }

    private Path[] vPathsToArrPaths(Vector vPaths){
	Path[] arrPaths = new Path[vPaths.size()];
	for(int i=0;i<vPaths.size();i++){
	    arrPaths[i] = (Path)vPaths.elementAt(i);
	}
	return arrPaths;
    }

    private Cycle[] vCyclesToArrCycles(Vector vCycle){
	Cycle[] arrCycle = new Cycle[vCycle.size()];
	for(int i=0;i<vCycle.size();i++){
	    arrCycle[i] = (Cycle)vCycle.elementAt(i);
	}
	return arrCycle;
    }


    //POST: RES is null if there's no edge to an unvisited nhbr otherwise, it's an edge to an unvisited nhbr.
    private RGEdge getEdgeToUnvisitedNhbr(boolean arrVisited[], RGNode rgn){
	RGEdge arrRGE[] = rgn.getEdges();
	int numEdges = arrRGE.length;
	//Debug.trace(numEdges);
	//Debug.trace(rgn);
	Assert.PRE(numEdges<=2);
	RGEdge rgeRes = null;
	for(int i=0;i<numEdges;i++){
	    RGEdge currEdge = arrRGE[i];
	    RGNode nhbr = currEdge.getNeighbor(rgn);
	    if(!arrVisited[nhbr.getUniqueNumber()]){
		rgeRes = currEdge;
	    }
	}
	return rgeRes;
    }


    RGEdge getEdgeBetween(RGNode rgn1, RGNode rgn2){
	RGEdge arrRGE[] = rgn1.getEdges();
	int numEdges = arrRGE.length;
	Assert.PRE(numEdges<=2);
	RGEdge rgeRes = null;
	for(int i=0;i<numEdges;i++){
	    RGEdge currEdge = arrRGE[i];
	    RGNode nhbr = currEdge.getNeighbor(rgn1);
	    if(nhbr==rgn2){
		return currEdge;
	    }
	}
	Assert.POST(false);
	return null;
    }

    //POST: RES is all the edges in the connected component containing
    //firsNode.  these edges are obtained by doing a dfs from first
    //node

    private RGEdge[] getCCEdges(int numNodesInGraph, RGNode firstNode){
	RGNode rgCurr = firstNode;
	Vector vEdge = new Vector();
	boolean done = false;
	boolean arrVisited[] = MyArrayUtils.getAllFalseArr(numNodesInGraph);

	while(!done){
	    int currID = rgCurr.getUniqueNumber();
	    arrVisited[currID] = true;
	    RGEdge nextRGE = getEdgeToUnvisitedNhbr(arrVisited,rgCurr);
	    if(nextRGE==null){
		//vEdge.addElement(getEdgeBetween(rgCurr,firstNode));
		done = true;
	    }else{
		vEdge.addElement(nextRGE);
		rgCurr = nextRGE.getNeighbor(rgCurr);
	    }
	}
	RGEdge arrRGE[] = rgCurr.getEdges();
	for(int i=0;i<arrRGE.length;i++){
	    RGEdge rge = arrRGE[i];
	    RGNode nhbr = rge.getNeighbor(rgCurr);
	    if(nhbr==firstNode && rge!=vEdge.elementAt(0)){
		vEdge.addElement(rge);
	    }
	}
	return vEdgeToarrRGEdge(vEdge);
    }

    private RGEdge[] vEdgeToarrRGEdge(Vector vEdge){
	int vSize = vEdge.size();
	RGEdge arrRGE[] = new RGEdge[vSize];
	for(int i=0;i<vSize;i++){
	    arrRGE[i] = (RGEdge)vEdge.elementAt(i);
	}
	return arrRGE;
    }

    //PRE: the node containt rgn on the left side is matched if leftP
    //is true else the node containing rgn on the right side is matched
    //POST: let n be the node which pairs with rgn on the left side if leftP is true or else on the right side of match.
    //then RES is the edge in match for the node n or null if n is not matched up.
//     private RGEdge getEdge(RGNode rgn, Matching match, RGNtoBNMapper rgnTobn, boolean leftP){
// 	RGEdge rge = null;

// 	Node node = null;
// 	if(leftP){
// 	    node = rgnTobn.getLeftNodeFromRGN(rgn);
// 	}else{
// 	    node = rgnTobn.getRightNodeFromRGN(rgn);
// 	}
// 	if(node!=null){
// 	    Edge edge = match.getMatchingEdge(node);
// 	    RGNode rgnSource = (edge.source().rgnode());
// 	    RGNode rgnSink = (edge.sink().rgnode());
// 	    rge = new RGEdge(rgnSource,rgnSink);
// 	}
// 	return rge;
//     }


    //PRE: no node in the edges of arrRGE is marked visited
    //POST: arrIsVisited is changed so that all nodes in arrRGE are marked visited.
    private boolean []markVisited(RGEdge arrRGE[], boolean arrIsVisited[],RGNode firstNode){
	boolean arrRes[] = MyArrayUtils.copyBoolArr(arrIsVisited);

	int firstID = firstNode.getUniqueNumber();
	Assert.PRE(!(arrIsVisited[firstID]));
	arrRes[firstID] = true;
	RGNode currNode = firstNode;
	int numEdges = arrRGE.length;
	for(int i=0;i<numEdges;i++){
	    RGEdge currEdge = arrRGE[i];
	    currNode = currEdge.getNeighbor(currNode);
	    int currID = currNode.getUniqueNumber();
	    if(currID==firstID){
		Assert.PRE(i==numEdges-1);
	    }else{
		Assert.PRE(arrRes[currID]==false);
		arrRes[currID] = true;
	    }
	}
	return arrRes;
    }

    public void scheduleEdges(Path arrPaths[], Cycle arrCycle[], Stack stFreeBypass){
	Vector vEdgeSend = new Vector();
	Vector vEdgeAdd = new Vector();
	Vector vEdgeBypassed = new Vector();

	Assert.PRE(arrPaths!=null);
	Assert.PRE(arrCycle!=null);
	for(int i=0;i<arrPaths.length;i++){
	    Path currPath = arrPaths[i];
	    addEvenEdgesToVector(vEdgeSend,currPath.getEdges());
	}
	
	//note: no node in a cycle can have a bypass machine which wants to send to it.
	for(int i=0;i<arrCycle.length;i++){
	    Cycle currCycle = arrCycle[i];
	    if (currCycle.isEven()){
		addEvenEdgesToVector(vEdgeSend,currCycle.getEdges());
	    }else{ //currCycLength is odd
		RGEdge bypassedEdge = currCycle.getBypassedEdge();
		vEdgeBypassed.addElement(bypassedEdge);

		RGEdge arrEdges[] = currCycle.getEdgesToSend();
		addEdgesToVector(vEdgeSend,arrEdges);

		RGEdge rgeBypass = getBypassEdge(bypassedEdge,stFreeBypass);
		vEdgeAdd.addElement(rgeBypass);
	    }
	}
	arrAddEdges = RGEdge.vectorToarrEdge(vEdgeAdd);
	arrSendEdges = RGEdge.vectorToarrEdge(vEdgeSend);
	arrBypassedEdges = RGEdge.vectorToarrEdge(vEdgeBypassed);
    }

    //PRE: rgn is the node that needs to bypass, rge is the edge that is bypassed.
    //POST: RES is the new edge that is added after the bypass is done.
    private RGEdge getBypassEdge(RGEdge rge, Stack stFreeBypass){
	Assert.PRE(rge!=null);
	RGNode rgnSender = rge.source();
	RGNode rgnReceiver = rge.sink();
	Assert.PRE(!stFreeBypass.empty());
	RGNode rgnBypass = (RGNode)stFreeBypass.pop();
	RGEdge rgeBypassEdge = new RGEdge(rgnBypass,rgnReceiver);
	return rgeBypassEdge;
    }




    public String toString(){
	return "";
    }
    
    public RGEdge[] getAddEdges(){
	return arrAddEdges;
    }

    public RGEdge[] getSendEdges(){
	return arrSendEdges;
    }

    public RGEdge[] getBypassedEdge(){
	return arrBypassedEdges;
    }
    
    public RGEdge[] getRemoveEdges(){
	int numRemEdges = arrSendEdges.length + arrBypassedEdges.length;
	RGEdge arrRem[] = new RGEdge[numRemEdges];
	int i;
	for(i=0;i<arrSendEdges.length;i++){
	    arrRem[i] = arrSendEdges[i];
	}
	for(int j=0;j<arrBypassedEdges.length;j++){
	    arrRem[i] = arrBypassedEdges[j];
	    i++;
	}
	return arrRem;
    }

    //POST: RES is true iff ids of all sources in arrRGE are false in arrRGE
    private boolean addSourcesArrUsed(RGEdge arrRGE[], boolean arrUsed[]){
	for(int i=0;i<arrRGE.length;i++){
	    RGNode rgn = arrRGE[i].source();
	    int id = rgn.getUniqueNumber();
	    if(arrUsed[id]){
		return false;
	    }else{
		arrUsed[id] = true;
	    }
	}
	return true;
    }

    //POST: RES is true iff ids of all sources in arrRGE are false in arrRGE
    private boolean addSinksArrUsed(RGEdge arrRGE[], boolean arrUsed[]){
	for(int i=0;i<arrRGE.length;i++){
	    RGNode rgn = arrRGE[i].sink();
	    int id = rgn.getUniqueNumber();
	    if(arrUsed[id]){
		return false;
	    }else{
		arrUsed[id] = true;
	    }
	}
	return true;
    }


    //PRE: numNodes is one greater than the maximum id of a node in arrEdges
    //POST: RES is true iff no endpoint of an edge in arrEdges appears twice
    private boolean isValid(int numNodes){
	boolean res = true;
	boolean arrUsed[] = MyArrayUtils.getAllFalseArr(numNodes);

	res = res && addSourcesArrUsed(arrBypassedEdges,arrUsed);
	res = res && addSourcesArrUsed(arrAddEdges,arrUsed);
	res = res && addSourcesArrUsed(arrSendEdges,arrUsed);
	res = res && addSinksArrUsed(arrSendEdges,arrUsed);	
	return res;
    }


    RGEdge arrBypassedEdges[];
    RGEdge arrAddEdges[];
    RGEdge arrSendEdges[];

};
