import java.io.*;
import java.util.*;

public class RGNode{

    public RGNode(int n){
	num = n;
	vOutEdges = new Vector();
	vInEdges = new Vector();
	pBypass = false;
    }

    public RGNode(int n, boolean isBypass){
	num = n;
	vOutEdges = new Vector();
	vInEdges = new Vector();
	pBypass = isBypass;
    }


    public int getUniqueNumber(){
	return num;
    }

    public void addEdge(RGEdge rge){
	RGNode source = rge.source();
	RGNode sink = rge.sink();
	if (source==this){
	    vOutEdges.addElement(rge);
	}
	else{
	    Assert.PRE(sink==this);
	    vInEdges.addElement(rge);
	}
    }

    //PRE: vEdge is a vector of RGEdges
    //POST: if rge is in vEdge, removes it from vEdge and returns true else returns false
    private static boolean removeEdgeFromVector(RGEdge rge, Vector vEdge){
	for(int i=0;i<vEdge.size();i++){
	    RGEdge currEdge = (RGEdge)vEdge.elementAt(i);
	    if(rge.equal(currEdge)){
		vEdge.removeElementAt(i);
		return true;
	    }
	}
	return false;
    }

    //PRE: rge is an edge adjacent to this
    public void removeEdge(RGEdge rge){
	RGNode source = rge.source();
	RGNode sink = rge.sink();
	if(source==this){
	    boolean prem = removeEdgeFromVector(rge,vOutEdges);
	    Assert.PRE(prem);
	}else{
	    Assert.PRE(sink==this);
	    boolean prem = removeEdgeFromVector(rge,vInEdges);
	    Assert.PRE(prem);
	}
    }

    public RGEdge[] getOutEdges(){
	RGEdge arrRGE[] = RGEdge.vectorToarrEdge(vOutEdges);
	return arrRGE;
    }

    public RGEdge[] getEdges(){
	int degree = this.degree();
	RGEdge arrRGE[] = new RGEdge[degree];
	int i;
	for(i=0;i<vInEdges.size();i++){
	    arrRGE[i] = (RGEdge) vInEdges.elementAt(i);
	}
	for(;i<degree;i++){
	    int outIndex = i - vInEdges.size();
	    arrRGE[i] = (RGEdge) vOutEdges.elementAt(outIndex);
	}
	return arrRGE;
    }



    public String toString(){
	String s = "Node: " + Integer.toString(num);
	s = s + isBypass() + "\n";

	s += " Num Out Edges: " + vOutEdges.size() + "\n";
	for(int i=0;i<vOutEdges.size();i++){
	    RGEdge outEdge = (RGEdge)vOutEdges.elementAt(i);
	    RGNode nNeighbor = outEdge.sink();
	    int outNum = nNeighbor.getUniqueNumber();
	    s = s + Integer.toString(outNum) + " ";
	}

	s = s + "Num In Edges " + vInEdges.size() + "\n";
	for(int i=0;i<vInEdges.size();i++){
	    RGEdge inEdge = (RGEdge)vInEdges.elementAt(i);
	    RGNode nNeighbor = inEdge.source();
	    int inNum = nNeighbor.getUniqueNumber();
	    s = s + Integer.toString(inNum) + " ";
	}

	return s;
    }

    public int degree(){
	int d = vOutEdges.size() + vInEdges.size();
	return d;
    }

    boolean isBypass(){
	return pBypass;
    }


    Vector vOutEdges;
    Vector vInEdges;
    int num;
    boolean pBypass;
};
