import java.io.*;
import java.util.*;


//a request graph.  This may be a multigraph.  One copy is stored of
//every unique edge with the number of occurences of that edge.

public class RGraph{

    RGraph(){
	int numNodes = 5;
	arrNodes = new RGNode[numNodes];
	arrEdges = new RGEdge[numNodes+1];
	for(int i=0;i<numNodes;i++){
	    arrNodes[i] = new RGNode(i);
	}
	addEdge(0,2,0);
	addEdge(0,3,1);
	addEdge(0,4,2);
	addEdge(1,2,3);
	addEdge(1,3,4);
	addEdge(1,4,5);

	//maxDegree = computeMaxDegree();
    }

    private int getMaxNodeNum(Vector vRGEdges){
	int vSize = vRGEdges.size();
	int maxNum = 0;
	for(int i=0;i<vSize;i++){
	    RGEdge rge = (RGEdge) vRGEdges.elementAt(i);
	    maxNum = MyMath.max(maxNum,rge.source.getUniqueNumber());
	    maxNum = MyMath.max(maxNum,rge.sink.getUniqueNumber());
	}
	return maxNum;
    }

    public RGraph(Vector vRGEdges){
	int maxNodeNum = getMaxNodeNum(vRGEdges);
	int numNodes = maxNodeNum + 1;
	int numEdges = vRGEdges.size();
	this.arrNodes = new RGNode[numNodes];
	this.arrEdges = new RGEdge[numEdges];

	for(int i=0;i<numNodes;i++){
	    arrNodes[i] = new RGNode(i);
	}

	for(int i=0;i<numEdges;i++){
	    RGEdge rge = (RGEdge)vRGEdges.elementAt(i);
	    int sourceid = rge.source.getUniqueNumber();
	    int sinkid = rge.sink.getUniqueNumber();
	    addEdge(sourceid, sinkid,i);
	}
    }


    //POST: this is  a random multigraph with numNodes nodes and numEdges edges
    //the edges are added with uniform probability over all possible edges
    public RGraph(int numNormalNodes, int numEdges, Random rSource){
	//Random rSource = new Random(107);
	int numNodesTotal = (3*numNormalNodes)/2;
	stNFreeBypass = new Stack();

	arrNodes = new RGNode[numNodesTotal];
	arrEdges = new RGEdge[numEdges];
	for(int i=0;i<numNormalNodes;i++){
	    arrNodes[i] = new RGNode(i);
	}
	for(int i=numNormalNodes;i<numNodesTotal;i++){
	    arrNodes[i] = new RGNode(i,true);
	    stNFreeBypass.push(arrNodes[i]);
	}

	for(int i=0;i<numEdges;i++){
	    int n1 = MyMath.randomNumber(0,numNormalNodes,rSource);
	    int n2 = MyMath.diffRandomNumber(0,numNormalNodes,n1,rSource);
	    addEdge(n1,n2,i);
	}
    }


    public int computeMaxDegree(){
	int maxDegree = 0;
	for(int i=0;i<arrNodes.length;i++){
	    RGNode currN = arrNodes[i];
	    int currDegree = currN.degree();
	    maxDegree = MyMath.max(maxDegree,currDegree);
	}
	return maxDegree;
    }


    //PRE: n1, n2 are between 0 and numNodes.  edgeNum is between 0 and numEdges
    //POST: adds the edge between the node numbered n1 and the node numbered n2, puts this edge
    private void addEdge(int n1,int n2, int edgeNum){
	int numNodes = arrNodes.length;
	int numEdges = arrEdges.length;
	Assert.INRANGE(n1,0,numNodes);
	Assert.INRANGE(n2,0,numNodes);
	Assert.INRANGE(edgeNum,0,numEdges);

	RGNode source = arrNodes[n1];
	RGNode sink = arrNodes[n2];
	Assert.PRE(source!=null);
	Assert.PRE(sink!=null);

	RGEdge rge = new RGEdge(source,sink);
	Assert.PRE(arrEdges!=null);
	arrEdges[edgeNum] = rge;
	
	source.addEdge(rge);
	sink.addEdge(rge);
    }


    public String toString(){
	String s = MyIOUtils.arrToString(arrNodes,"\n");
	return s;
    }
    
    public RGNode[] getNodes(){
	return arrNodes;
    }

    public int numNodes(){
	return arrNodes.length;
    }

    public RGEdge[] getEdges(){
	return arrEdges;
    }

    public int maxDegree(){
	return computeMaxDegree();
    }

    public Stack getFreeBypassNodes(){
	return stNFreeBypass;
    }


    private RGNode getNodeWithSameID(RGNode rgn){
	int rgnid = rgn.getUniqueNumber();
	return arrNodes[rgnid];
    }
	

    //PRE: the edges in arrEAdd and arrERemove have endpoints that are
    //all in the graph.  The edges in arrERemove are all in the graph
    //and the edges in arrERemove all have one endpoint which is a bypass node

    //POST: edges in arrEAdd are added to the graph and edges in
    //arrERemove are removed from the graph.
    public void updateEdges(RGEdge[] arrEAdd, RGEdge[] arrERemove){

	for(int i=0;i<arrERemove.length;i++){
	    RGEdge rgeCurrRem = arrERemove[i];
	    RGNode source = rgeCurrRem.source();
	    RGNode sink = rgeCurrRem.sink();

	    source = getNodeWithSameID(source);
	    sink = getNodeWithSameID(sink);
	    rgeCurrRem = new RGEdge(source, sink);

	    source.removeEdge(rgeCurrRem);
	    sink.removeEdge(rgeCurrRem);
	}
	for(int i=0;i<arrEAdd.length;i++){
	    RGEdge rgeCurrAdd = arrEAdd[i];
	    RGNode source = rgeCurrAdd.source();
	    RGNode sink = rgeCurrAdd.sink();

	    source = getNodeWithSameID(source);
	    sink = getNodeWithSameID(sink);

	    rgeCurrAdd = new RGEdge(source,sink);
	    source.addEdge(rgeCurrAdd);
	    sink.addEdge(rgeCurrAdd);
	}
	stNFreeBypass = getStackOfFreeBypassNodes(arrNodes);
	arrEdges = getAllEdges(arrNodes);
    }

    Stack getStackOfFreeBypassNodes(RGNode arrNodes[]){
	Stack stRGN = new Stack();
	for(int i=0;i<arrNodes.length;i++){
	    RGNode rgn = arrNodes[i];
	    if(rgn.degree()==0&&rgn.isBypass()){
		stRGN.push(rgn);
	    }
	}
	return stRGN;
    }

    RGEdge []getAllEdges(RGNode arrNodes[]){

	Vector vEdges = new Vector();
	for(int i=0;i<arrNodes.length;i++){
	    RGNode rgn = arrNodes[i];
	    RGEdge arrRGE[] = rgn.getOutEdges();
	    for(int j=0;j<arrRGE.length;j++){
		vEdges.addElement(arrRGE[j]);
	    }
	}
	RGEdge arrRGE[] = RGEdge.vectorToarrEdge(vEdges);
	return arrRGE;
    }

    private static RGNode findNode(int id, RGNode arrNCandidate[]){
	for(int i=0;i<arrNCandidate.length;i++){
	    RGNode rgn = arrNCandidate[i];
	    if(rgn.getUniqueNumber()==id){
		return rgn;
	    }
	}
	Assert.POST(false);
	return null;
    }


    //PRE: for every node in rg n1, there is a node in arrNCandidate n2 which has the same id
    //POST: RES is an array of all the nodes in arrNCandidate which
    //have id's the same as nodes in rg that have maxdegree
    public static RGNode[] getMaxNodes(RGNode arrOrig[], RGNode arrNCandidate[], int maxDegree){
	//int maxDegree = rg.computeMaxDegree();
	Vector vRes = new Vector();
	///RGNode arrNodes[] = rg.getNodes();
	//Assert.PRE(arrOrig.length==arrNCandidate.length);
	for(int i=0;i<arrOrig.length;i++){
	    RGNode rgn = arrOrig[i];
	    if(rgn.degree()==maxDegree){
		RGNode rgnCand = findNode(rgn.getUniqueNumber(),arrNCandidate);
		vRes.addElement(rgnCand);
	    }
	}
	RGNode arrRes[] = new RGNode[vRes.size()];
	for(int i=0;i<vRes.size();i++){
	    arrRes[i] = (RGNode)vRes.elementAt(i);
	}
	return arrRes;
    }
	    


    Stack stNFreeBypass;
    RGNode arrNodes[];
    RGEdge arrEdges[];
};
